import { X509Certificate } from 'crypto';
import * as fs from 'fs';
import path from 'path';
import _ from 'lodash';

class Cert {
  public cert = new X509Certificate(
    fs.readFileSync(path.join('/etc/nginx/certs', this.dir, 'fullchain.pem')),
  );
  constructor(public dir: string) {}

  isNotExpired() {
    const now = new Date();
    const validFrom = new Date(this.cert.validFrom);
    const validTo = new Date(this.cert.validTo);
    return now > validFrom && now < validTo;
  }

  isOkWithDomain(domain: string) {
    return this.cert.checkHost(domain);
  }

  isOkWithDomains(domains: string[]) {
    return domains.every((domain) => this.isOkWithDomain(domain));
  }
}

const certs = fs
  .readdirSync('/etc/nginx/certs')
  .filter((dir) =>
    fs.existsSync(path.join('/etc/nginx/certs', dir, 'fullchain.pem')),
  )
  .map((dir) => new Cert(dir))
  .filter((cert) => cert.isNotExpired());

export function pickCert(domains: string[]) {
  const okCerts = certs.filter((cert) => cert.isOkWithDomains(domains));
  if (!okCerts.length) {
    return;
  }
  return _.maxBy(okCerts, (cert) => cert.cert.validTo).dir;
}
